@@ -10,23 +10,23 @@ our @CARP_NOT = ('Rstats');
 
 use overload
   bool => \&bool,
-  '+' => sub { shift->operation('add', @_) },
-  '-' => sub { shift->operation('subtract', @_) },
-  '*' => sub { shift->operation('multiply', @_) },
-  '/' => sub { shift->operation('divide', @_) },
-  '%' => sub { shift->operation('remainder', @_) },
+  '+' => sub { shift->operate_binary(\&Rstats::VectorFunc::add, @_) },
+  '-' => sub { shift->operate_binary(\&Rstats::VectorFunc::subtract, @_) },
+  '*' => sub { shift->operate_binary(\&Rstats::VectorFunc::multiply, @_) },
+  '/' => sub { shift->operate_binary(\&Rstats::VectorFunc::divide, @_) },
+  '%' => sub { shift->operate_binary(\&Rstats::VectorFunc::remainder, @_) },
   'neg' => sub { shift->negation(@_) },
-  '**' => sub { shift->operation('pow', @_) },
+  '**' => sub { shift->operate_binary(\&Rstats::VectorFunc::pow, @_) },
   'x' => sub { shift->inner_product(@_) },
-  '<' => sub { shift->operation('less_than', @_) },
-  '<=' => sub { shift->operation('less_than_or_equal', @_) },
-  '>' => sub { shift->operation('more_than', @_) },
-  '>=' => sub { shift->operation('more_than_or_equal', @_) },
-  '==' => sub { shift->operation('equal', @_) },
-  '!=' => sub { shift->operation('not_equal', @_) },
+  '<' => sub { shift->operate_binary(\&Rstats::VectorFunc::less_than, @_) },
+  '<=' => sub { shift->operate_binary(\&Rstats::VectorFunc::less_than_or_equal, @_) },
+  '>' => sub { shift->operate_binary(\&Rstats::VectorFunc::more_than, @_) },
+  '>=' => sub { shift->operate_binary(\&Rstats::VectorFunc::more_than_or_equal, @_) },
+  '==' => sub { shift->operate_binary(\&Rstats::VectorFunc::equal, @_) },
+  '!=' => sub { shift->operate_binary(\&Rstats::VectorFunc::not_equal, @_) },
+  '&' => sub { shift->operate_binary(\&Rstats::VectorFunc::and, @_) },
+  '|' => sub { shift->operate_binary(\&Rstats::VectorFunc::or, @_) },
   '""' => sub { shift->to_string(@_) },
-  '&' => sub { shift->operation('and', @_) },
-  '|' => sub { shift->operation('or', @_) },
   fallback => 1;
 
 sub to_string {
@@ -213,12 +213,9 @@ sub is_nan {
 sub is_null {
   my $x1 = Rstats::Func::to_c(shift);
   
-  my @a2_elements = [!$x1->length_value ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE()];
-  my $x2 = Rstats::Func::c(\@a2_elements);
-  $x1->copy_attrs_to($x1);
-  $x2->mode('logical');
+  my $x_is = $x1->length_value == 0 ? Rstats::Func::TRUE() : Rstats::Func::FALSE();
   
-  return $x2;
+  return $x_is;
 }
 
 sub getin { shift->get(@_) }
@@ -259,7 +256,7 @@ sub get {
   # array
   my $x2 = Rstats::Func::array(
     Rstats::Func::new_vector($self->vector->type, @a2_values),
-    $x2_dim
+    Rstats::ArrayFunc::c(@$x2_dim)
   );
   
   # Copy attributes
@@ -295,15 +292,15 @@ sub set {
   
   my $self_elements;
   if ($self->is_factor) {
-    $self_elements = $self->decompose_elements;
+    $self_elements = $self->decompose;
     $x2 = $x2->as_character unless $x2->is_character;
-    my $x2_elements = $x2->decompose_elements;
+    my $x2_elements = $x2->decompose;
     my $levels_h = $self->_levels_h;
     for (my $i = 0; $i < @$poss; $i++) {
       my $pos = $poss->[$i];
       my $element = $x2_elements->[(($i + 1) % @$poss) - 1];
       if ($element->is_na->value) {
-        $self_elements->[$pos] = Rstats::VectorFunc::NA();
+        $self_elements->[$pos] = Rstats::VectorFunc::new_logical(undef);
       }
       else {
         my $value = $element->to_string;
@@ -312,7 +309,7 @@ sub set {
         }
         else {
           carp "invalid factor level, NA generated";
-          $self_elements->[$pos] = Rstats::VectorFunc::NA();
+          $self_elements->[$pos] = Rstats::VectorFunc::new_logical(undef);
         }
       }
     }
@@ -326,9 +323,9 @@ sub set {
       $self->vector($self_tmp->vector);
     }
 
-    $self_elements = $self->decompose_elements;
+    $self_elements = $self->decompose;
 
-    my $x2_elements = $x2->decompose_elements;
+    my $x2_elements = $x2->decompose;
     for (my $i = 0; $i < @$poss; $i++) {
       my $pos = $poss->[$i];
       $self_elements->[$pos] = $x2_elements->[(($i + 1) % @$poss) - 1];
@@ -351,58 +348,60 @@ sub bool {
     carp 'In if (a) { : the condition has length > 1 and only the first element will be used';
   }
   
-  my $element = $self->vector_part;
+  my $type = $self->type;
+  my $value = $self->value;
 
   my $is;
-  if ($element->is_character || $element->is_complex) {
+  if ($type eq 'character' || $type eq 'complex') {
     croak 'Error in -a : invalid argument to unary operator ';
   }
-  elsif ($element->is_double) {
-    if ($element->is_infinite->value) {
+  elsif ($type eq 'double') {
+    if ($value eq 'Inf' || $value eq '-Inf') {
       $is = 1;
     }
-    elsif ($element->is_nan->value) {
+    elsif ($value eq 'NaN') {
       croak 'argument is not interpretable as logical';
     }
     else {
-      $is = $element->value;
+      $is = $value;
     }
   }
-  elsif ($element->is_integer || $element->is_logical) {
-    $is = $element->value;
+  elsif ($type eq 'integer' || $type eq 'logical') {
+    $is = $value;
   }
   else {
     croak "Invalid type";
   }
   
-  my $is_na = $element->is_na->value;
-  if ($is_na) {
+  if (!defined $value) {
     croak "Error in bool context (a) { : missing value where TRUE/FALSE needed"
   }
 
   return $is;
 }
 
-sub vector_part {
+sub value {
   my $self = shift;
 
+  my $e1;
   my $dim_values = $self->dim_as_array->values;
-  
-  my $self_elements = $self->decompose_elements;
+  my $self_elements = $self->decompose;
   if (@_) {
     if (@$dim_values == 1) {
-      return $self_elements->[$_[0] - 1];
+      $e1 = $self_elements->[$_[0] - 1];
     }
     elsif (@$dim_values == 2) {
-      return $self_elements->[($_[0] + $dim_values->[0] * ($_[1] - 1)) - 1];
+      $e1 = $self_elements->[($_[0] + $dim_values->[0] * ($_[1] - 1)) - 1];
     }
     else {
-      return $self->get(@_)->decompose_elements->[0];
+      $e1 = $self->get(@_)->decompose->[0];
     }
   }
   else {
-    return $self_elements->[0];
+    $e1 = $self_elements->[0];
   }
+  
+  return defined $e1 ? $e1->value : undef;
 }
 
 sub inner_product {
@@ -416,13 +415,13 @@ sub inner_product {
 
 sub negation { Rstats::Func::negation(@_) }
 
-sub operation {
-  my ($self, $op, $data, $reverse) = @_;
+sub operate_binary {
+  my ($self, $func, $data, $reverse) = @_;
   
   # fix postion
   my ($x1, $x2) = $self->_fix_position($data, $reverse);
   
-  return Rstats::Func::operation($op, $x1, $x2);
+  return Rstats::Func::operate_binary($func, $x1, $x2);
 }
 
 sub _fix_position {
@@ -436,12 +435,12 @@ sub _fix_position {
   }
   else {
     if ($reverse) {
-      $x1 = Rstats::Func::c($data);
+      $x1 = Rstats::ArrayFunc::c($data);
       $x2 = $self;
     }
     else {
       $x1 = $self;
-      $x2 = Rstats::Func::c($data);
+      $x2 = Rstats::ArrayFunc::c($data);
     }
   }
   
